/**************************************************************************
 *   kn_arp.c                                                             *
 *                                                                        *
 *   Copyright (C) 2003-2005 Halil Demirezen  <halil@enderunix.org>       *
 *                                                                        *
 *   kn_arp.c includes functions that send and recieve ARP packets to     *
 *   extract IP and MAC address infos of the online machines on the LAN.  *
 *                                                                        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 2, or (at your option)  *
 *   any later version.                                                   *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, write to the Free Software          *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
 *                                                                        *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <libnet.h>

#include "kn_defs.h"

extern char *interface;
extern int output_mode;

bpf_u_int32 netmask(char *interface)
{
    bpf_u_int32 netmask;
    bpf_u_int32 ipaddr;
    char *dev;
    char errbuf[PCAP_ERRBUF_SIZE];

    dev = interface;
    pcap_lookupnet(dev, &ipaddr, &netmask, errbuf);
    return(netmask);
}

#ifdef __LIBNET_1_1__
void sd_arp_thr()
{
	u_char errbuf[256];
        libnet_t *lnet;
        libnet_ptag_t arp_tg, eth_tg;
        struct libnet_ether_addr *eth;
        u_long number, i, ipaddr,net;
        u_long ip;
        char *dev;
        u_int8_t *packet;
        u_int32_t packet_s;
        unsigned char b1[6];
        unsigned char b2[6];


        dev = interface;
        lnet = libnet_init(LIBNET_LINK_ADV, dev, errbuf);
        if(lnet == NULL){
                fprintf(stderr, "LIBNET: %s\n", errbuf);
                exit(-1);
        }

        ipaddr = libnet_get_ipaddr4(lnet);
        eth = libnet_get_hwaddr(lnet);
        net = netmask(dev);

        memset(&b1, 0xff, 6);
        memset(&b2, 0x00, 6);
        number = ntohl(~net);

	for(i = 1; i < number; i++) {
                ip = (ipaddr & net) | htonl(i);
                arp_tg = libnet_autobuild_arp(
                                ARPOP_REQUEST,
                                eth->ether_addr_octet,
                                (u_int8_t *)&ipaddr,
                                b2,
                                (u_int8_t *)&ip,
                                lnet);
                eth_tg = libnet_autobuild_ethernet(
                                b1,
                                ETHERTYPE_ARP,
                                lnet);
                if(libnet_adv_cull_packet(lnet, &packet, &packet_s) == -1){
                        fprintf(stderr, "LIBNET: %s\n", libnet_geterror(lnet));
                        exit(-1);
                }
                libnet_adv_free_packet(lnet, packet);
                if(libnet_write(lnet) == -1){
                        fprintf(stderr, "LIBNET: %s\n", libnet_geterror(lnet));
                        exit(-1);
                }
                libnet_pblock_delete(lnet, libnet_pblock_find(lnet, arp_tg));
                libnet_pblock_delete(lnet, libnet_pblock_find(lnet, eth_tg));
                usleep(500);
        }
	sleep(1);
	exit(0);
}
#else
#include <netinet/if_ether.h>
#ifdef __BSD__
#define ether_addr_octet octet
#endif
void sd_arp_thr()
{
	struct arppacket *arp;
        u_short packet_size = 42;
        char errbuf[256];
        unsigned ip, ip_dst,net;
        u_char *packet;
        struct ether_addr *eth;
	struct libnet_link_int *libnet_link_int;
        u_long number, i;
        char *dev;

        if(libnet_init_packet(packet_size, &packet) == -1){
                printf("Can't allocate memory for packet\n");
        	exit(-1);
	}

        dev = interface;
        libnet_init_packet(packet_size, &packet);
        if ((libnet_link_int = (struct ether_addr *)libnet_open_link_interface(dev, (char *)&errbuf)) == NULL) {
                fprintf(stderr, "cannot open link interface %s: %s\n", dev, errbuf);
       		exit(-1); 
	}

        ip = libnet_get_ipaddr(libnet_link_int, dev, errbuf);
        ip = htonl(ip);
        eth = (struct libnet_ether_addr *)libnet_get_hwaddr(libnet_link_int, dev, errbuf);

	arp = (struct arppacket *)packet;
        arp->protocoltype = htons(ETHERTYPE_ARP);    /* ARP protocol */
        arp->hardwaretype = htons(ARPHRD_ETHER);     /* Hardware ether */
        arp->resprotocol = htons(ETHERTYPE_IP);      /* Next protocol */
        arp->halen = 6;                              /* Hardare address length */
        arp->palen = 4;                              /* protocol address length */
        arp->opcode = htons(ARPOP_REQUEST);          /* Arp Opcode */
        memset(arp->destmac, 0xff, 6);
        memset(arp->destadd, 0x0, 6);
        memcpy(arp->sourcemac, eth->ether_addr_octet, 6);   /* my ethernet mac */
        memcpy(arp->sourceadd, eth->ether_addr_octet, 6);   /* my arp mac */
        memcpy(arp->sourceip, (char *)&ip, 4);              /* my arp ip address */
        net = netmask(dev);                           /* let's get mask */
        number = ntohl(~net);                               /* maximum number of hosts in the lan */
        
	for(i = 1; i < number; i++) {
		ip_dst = (ip & net) | htonl(i);
                memcpy(arp->destip, (char *)&ip_dst, 4);
                if(libnet_write_link_layer(libnet_link_int, dev, (u_char *)packet, packet_size) == -1){
                        printf("libnet_write_link_layer failed\n");
               		exit(-1); 
		}
                usleep(500);
        }
	sleep(1);
	exit(0);
}
#endif

void rc_arp_thr()
{
	char *dev;
        char errbuf[PCAP_ERRBUF_SIZE];
        pcap_t *descr;
        const u_char *packet;
        struct pcap_pkthdr hdr;     /* pcap.h */
       	struct libnet_ethernet_hdr *eth;
        struct arppacket *arp;
        struct in_addr tmp;
        unsigned long ip;
        unsigned char buffer[256];

        dev = interface;

        if ((descr = pcap_open_live(dev, 1500, 0, 500, errbuf)) == NULL) {
                fprintf(stderr, "pcap_open_live %s failed: %s\n", dev, errbuf);
                exit(1);
        }
        if(output_mode == NORMAL_OUTPUT){
                memset(buffer, 0x0, 256);
                printf("----------------------------------------------\n");
                printf(" EnderUNIX knowlan 1.0 - LAN Address Extractor \n");
                printf(" Use -h option for a brief help about the tool\n");
                printf("----------------------------------------------\n");
                printf("Interface Name: %s\n", dev);
	} 

	do{
                packet = pcap_next(descr,&hdr);
                eth = (struct ether_header *) packet;
                if (hdr.len < 14)
                        continue;
                if (eth == NULL)
                        continue;
                if(htons(eth->ether_type) == ETHERTYPE_ARP){
                        arp = (struct arppacket *) packet;
                        if(htons(arp->opcode) == ARPOP_REPLY){
                                memcpy((char *)&ip, arp->sourceip, 4);
                                tmp.s_addr = ip;
                                printf("%s\t%02X:%02X:%02X:%02X:%02X:%02X\n",
                                                inet_ntoa(tmp), arp->sourceadd[0],
                                                arp->sourceadd[1], arp->sourceadd[2],
                                                arp->sourceadd[3], arp->sourceadd[4],
                                                arp->sourceadd[5]);
                        }
                }
        }while(1);

} 	
