/**************************************************************************
 *   kn_main.c                                                            *
 *                                                                        *
 *   Copyright (C) 2003-2005 Halil Demirezen  <halil@enderunix.org>       *
 *                                                                        *
 *   kn_arp.c includes functions that send and recieve ARP packets to     *
 *   extract IP and MAC address infos of the online machines on the LAN.  *
 *                                                                        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 2, or (at your option)  *
 *   any later version.                                                   *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, write to the Free Software          *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
 *                                                                        *
 **************************************************************************/

#include <pthread.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <pcap.h>
#include <sys/wait.h>
#include "kn_defs.h"

extern void sd_arp_thr();
extern void rc_arp_thr();

void kn_help();
void kn_version();

int output_mode = 0;
char *interface = NULL;
char *optarg;

int main(int argc, char *argv[])
{
	pthread_attr_t attrs;
	pthread_t thread_id1, thread_id2;
	char errbuf[PCAP_ERRBUF_SIZE];
	int c;
	
	while((c = getopt(argc, argv, "i:hvq")) != -1){
		switch(c){
			case 'i':
				interface = optarg;
				break;
			case 'q':
				output_mode = QUIET_OUTPUT;
				break;
			case 'h':	
				kn_help();
				return 0;
			case 'v':
				kn_version();
				return 0;
		}
		printf("%c", c);
	}
	if(!interface){
		interface = pcap_lookupdev(errbuf);
		if(interface == NULL){
			fprintf(stderr, "can't open device\n");
			return -1;
		}
	} 	
	
	pthread_attr_init(&attrs);
	pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
       	pthread_attr_setscope(&attrs, PTHREAD_SCOPE_SYSTEM);
	pthread_create(&thread_id1, &attrs, (void *)&sd_arp_thr, (void *)0);
	pthread_create(&thread_id2, &attrs, (void *)&rc_arp_thr, (void *)0);
	for(;;);
}


void kn_help()
{
        fprintf(stderr, "Usage: knowlan [options] [interface]\n");
        fprintf(stderr, "options:\n");
        fprintf(stderr, "-i interface\tact on different interface\n");
        fprintf(stderr, "-h\t\tprints this help\n");
        fprintf(stderr, "-v\t\tversion number\n");
        fprintf(stderr, "-q\t\tA raw output for piping and logging\n");
        fprintf(stderr, "Halil Demirezen <halil@enderunix.org>\n");
}

void kn_version()
{
	  fprintf(stderr, "1.0-THR\n");
}

